--//////////SPI_Data_MUX /////////////////////////////////--
-- ***********************************************************************
-- FileName: SPI_Data_MUX.vhd
-- FPGA: Lattice MachXO2 1200ZE-1TG144C
-- IDE: Lattice Diamond ver. 1.4.2.105
--
-- HDL IS PROVIDED "AS IS." DIGI-KEY EXPRESSLY DISCLAIMS ANY
-- WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
-- PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL DIGI-KEY
-- BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
-- DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
-- PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
-- BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
-- ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
-- DIGI-KEY ALSO DISCLAIMS ANY LIABILITY FOR PATENT OR COPYRIGHT
-- INFRINGEMENT.
--
-- Version History
-- Version 1.0 15/06/2012 Tony Storey
-- Initial Public Release


LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.numeric_std.all;

-- Valid EFB SPI memory mapped register addresses are 0x54-0x5D
--  SPICR0			Control Reg 0							0x54				Read/Write
--  SPICR1			Control Reg1								0x55				Read/Write
--  SPICR2			Control Reg2								0x56				Read/Write
--  SPIBR				Clock Pre-Scale						0x57				Read/Write
--  SPICSR			Master Chip Select					0x58				Read/Write
--  SPITXDR			Transmit Data							0x59				Write
--  SPISR				Status											0x5A				Read
--  SPIRXDR			Receive Data								0x5B				Read
--  SPIIRQ				Interrupt Request					0x5C				Read/Write
--  SPIIRQEN		Interrupt Request Enable	0x5D				Read/Write

entity SPI_Data_MUX is
	Port(
				clk									: in std_logic;
				nreset							: in std_logic;															-- active low reset
				wb_busy						: in std_logic;															-- wishbone busy input signal
				reg_address				: in std_logic_vector(7 downto 0);				-- comes from recieve register
				QEI_0							: in std_logic_vector(7 downto 0);				--  from the respective QEI module can connect more endoder to the below ports
				QEI_1							: in std_logic_vector(7 downto 0);				-- not used 
				QEI_2							: in std_logic_vector(7 downto 0);				-- not used 
				QEI_3							: in std_logic_vector(7 downto 0);				-- not used 
				QEI_4							: in std_logic_vector(7 downto 0);				-- not used 
				Gray_code_0				: in std_logic_vector(7 downto 0);				-- from the gray code decoders
				Gray_code_1				: in std_logic_vector(7 downto 0);				-- not used 
				tct_sw							: in std_logic_vector(7 downto 0);				-- from the button inputs
				reg_address_o	: buffer std_logic_vector(7 downto 0);    			-- testing to get output from recieve reg
				wb_bus_en	: out std_logic;																	-- enable signal for wishbone bus controller
				w_enable		: out std_logic;																	-- output 0 for read and 1 for write
				tx_reg				: out  std_logic_vector(7 downto 0);						-- data to transmit back to master
				tx_address		: out  std_logic_vector(7 downto 0)							-- address to put the data or pull the data 0x59 for transmit buffer and 0x5B for reciever buffer and 0x5A for status reg
				);
end SPI_Data_MUX;



architecture arch of SPI_Data_MUX is
	-- encode for one hot fsm
	type state_type is (ready, RxRDY, get_data, get_data1, TxRDY, put_data);	
	-- THIS SECTION CAN BE UNCOMMENTED FOR ONE HOT ENCODING
	--attribute enum_encoding	: string;
	--attribute enum_encoding of state_type : type is "000001 000010 000100 001000 010000 100000";
	signal state_reg, state_next	: state_type;
	signal reg_address_ff	: std_logic_vector(7 downto 0);



begin

		-- STATE UPDATE AND TIMING WITH SYNCH ACTIVE LOW RESET
		process(clk, nreset)
		begin
			if(rising_edge(clk)) then
				if(nreset = '0') then
					state_reg <= ready;
				else
					state_reg <= state_next;
				end if;
			end if;
		end process;
		
			
								
		-- CONTROL LOGIC STATE MACHINE,  ADD TO SENSITIVITY LIST AS NEEDED WHEN MODIFIED
		process(state_reg,  reg_address, reg_address_ff, wb_busy,  QEI_0, Gray_code_0, tct_sw, reg_address_o)
		begin
			
			-- default conditions
			tx_address(7 downto 0) <= (others => 'Z');
			tx_reg(7 downto 0) <= (others => 'Z');

			-- testing
			wb_bus_en <= '0';

			case state_reg is
			
			
				-- initial state 
				------------------------------------------------------------------------------					
				when ready =>
				    wb_bus_en <= '0';

					tx_address(7 downto 0) <= (others => 'Z');
					tx_reg(7 downto 0) <= (others => 'Z');

					if( wb_busy = '1') then
						state_next <= state_reg;
					else
						state_next <= RxRDY;
					end if;
						
					
				--  read status reg to see if reciever contains a reg address yet	
				------------------------------------------------------------------------------	
				when RxRDY =>

					wb_bus_en <= '1';					
					w_enable	<= '0';		-- set to read the wb
					tx_address(7 downto 0) <= X"5A";
				
					if(wb_busy = '1') then
						state_next <= state_reg;
					else
						if(reg_address(3) = '1') then	
							state_next <= get_data;
						else
							state_next <= state_reg;
						end if;
					end if;


				-- This state is used to get the address data from the master
				------------------------------------------------------------------------------	
				when get_data =>
					
					wb_bus_en <= '0';
					w_enable <= '0';				-- set to read from reciever
			
					-- load wishbone reciever register
					tx_address(7 downto 0) <= X"5B";
	
					if(wb_busy= '1') then
						state_next <= state_reg;
					else
						state_next <= get_data1;
					end if;
		
		
				-- This state is used to latch in the address from the master into a temp register
				------------------------------------------------------------------------------	
				when get_data1 =>

					wb_bus_en <= '1';
					w_enable <= '0';				-- set to read from reciever

					
					tx_address(7 downto 0) <= X"5B";

					reg_address_ff <= reg_address;


					if( wb_busy = '1') then
						state_next <= state_reg;
					else
						state_next <= TxRDY;
					end if;

				-- This state checks the status of the spi transmitter to ensure its empty
				------------------------------------------------------------------------------	
				when TxRDY =>

					wb_bus_en <= '1';					
					w_enable	<= '0';		-- set to read the wb
					tx_address(7 downto 0) <= X"5A";

			
					if(wb_busy = '1') then
						state_next <= state_reg;
					else
						if(reg_address(4) = '1') then	
							state_next <= put_data;
						else
							state_next <= state_reg;
						end if;
					end if;

				-- This state sends the requested data to the spi transmit buffer
				------------------------------------------------------------------------------	
				when put_data =>
			
					wb_bus_en <= '1';
					w_enable <= '1';				-- set to read from reciever
				
					-- load wishbone transmitter register
					tx_address(7 downto 0) <= X"59";
				
					-- Add extra addresses and data lines to this if-elsif statement 
					-- when connecting more encoders
					if( reg_address_o = X"01") then
						tx_reg(7 downto 0) <=  QEI_0(7 downto 0);
					elsif( reg_address_o = X"02") then
						tx_reg(7 downto 0) <=  Gray_code_0(7 downto 0);
					elsif(reg_address_o = X"03") then
						tx_reg(7 downto 0) <=  not(tct_sw(7 downto 0));					
					end if;

					if( wb_busy = '1') then
						state_next <= state_reg;
					else
						state_next <= ready;
					end if;

			end case;		
			
	end process;
	
	
	
		-- Process to filter non register values
		-- this section can be commented out for quiet spi systems
		-- extra address values can be added or removed
		process(clk, nreset, reg_address_ff)
		begin
				
			if rising_edge(clk) then
				if(nreset = '0') then
					reg_address_o <= (others => '0');
				else
					if(reg_address_ff = X"01") then
						reg_address_o <= reg_address_ff;
					elsif(reg_address_ff = X"02") then
						reg_address_o <= reg_address_ff;
					elsif(reg_address_ff = X"03") then
						reg_address_o <= reg_address_ff;
					elsif(reg_address_ff = X"04") then
						reg_address_o <= reg_address_ff;
					elsif(reg_address_ff = X"05") then
						reg_address_o <= reg_address_ff;
					elsif(reg_address_ff = X"06") then
						reg_address_o <= reg_address_ff;
					elsif(reg_address_ff = X"07") then
						reg_address_o <= reg_address_ff;
					elsif(reg_address_ff = X"08") then
						reg_address_o <= reg_address_ff;
					elsif(reg_address_ff = X"09") then
						reg_address_o <= reg_address_ff;
					elsif(reg_address_ff = X"0A") then
						reg_address_o <= reg_address_ff;
					end if;
				end if;
			end if;
		end process;  

--	 reg_address_o <= reg_address_ff;


end arch;
