--//////////SPI_wbController /////////////////////////////////--
-- ***********************************************************************
-- FileName: SPI_wbController.vhd
-- FPGA: Lattice MachXO2 1200ZE-1TG144C
-- IDE: Lattice Diamond ver. 1.4.2.105
--
-- HDL IS PROVIDED "AS IS." DIGI-KEY EXPRESSLY DISCLAIMS ANY
-- WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
-- PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL DIGI-KEY
-- BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
-- DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
-- PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
-- BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
-- ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
-- DIGI-KEY ALSO DISCLAIMS ANY LIABILITY FOR PATENT OR COPYRIGHT
-- INFRINGEMENT.
--
-- Version History
-- Version 1.0 15/06/2012 Tony Storey
-- Initial Public Release


LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.numeric_std.all;

-- SPI Controller to interface wishbone bus with SPI

-- Valid EFB SPI memory mapped register addresses are 0x54-0x5D
--  SPICR0			Control Reg 0							0x54				Read/Write
--  SPICR1			Control Reg1								0x55				Read/Write
--  SPICR2			Control Reg2								0x56				Read/Write
--  SPIBR				Clock Pre-Scale						0x57				Read/Write
--  SPICSR			Master Chip Select					0x58				Read/Write
--  SPITXDR			Transmit Data							0x59				Write
--  SPISR				Status											0x5A				Read
--  SPIRXDR			Receive Data								0x5B				Read
--  SPIIRQ				Interrupt Request					0x5C				Read/Write
--  SPIIRQEN		Interrupt Request Enable	0x5D				Read/Write




entity SPI_wbController is
	Port(
				clk										: in  std_logic;
				nreset								: in  std_logic;												
				wb_bus_en					: in std_logic;													-- signal to enable the bus cycle
				wb_ack_o						: in  std_logic;
				wb_valid_address 		: in std_logic_vector(7 downto 0);
				wb_valid_data				: in std_logic_vector(7 downto 0);
				wb_valid_data_o			: in std_logic_vector(7 downto 0);
				w_enable						: in  std_logic;												-- inputs 0 for read and 1 for write
				busy						: out std_logic;
				wb_rst_i				: out  std_logic;
				wb_cyc_i				: out  std_logic; 
				wb_stb_i				: out  std_logic; 
				wb_we_i				: out  std_logic; 
				wb_adr_i				: out std_logic_vector(7 downto 0); 
				wb_dat_o			: out std_logic_vector(7 downto 0);
				wb_dat_i				: out  std_logic_vector(7 downto 0)
				);
end SPI_wbController;



architecture arch of SPI_wbController is
	-- encode for one hot fsm
	type state_type is (ready, edge_0, edge_1, edge_2);
	-- UNCOMMENT THIS SECTION FOR ONE-HOT EMPLIMENTATION
	--attribute enum_encoding	: string;
	--attribute enum_encoding of state_type : type is "0001 0010 0100 1000";
	signal state_reg, state_next	: state_type;

	
begin

		-- STATE UPDATE AND TIMING WITH SYNCH ACTIVE LOW RESET
		process(clk, nreset)
		begin
			if(rising_edge(clk)) then
				if(nreset = '0') then
					wb_rst_i <= '1';
					state_reg <= ready;
				else
					wb_rst_i <= '0';
					state_reg <= state_next;
				end if;
			end if;
		end process;
		
			

		-- CONTROL LOGIC STATE MACHINE
		process(state_reg, wb_ack_o, wb_valid_data, wb_valid_address, w_enable, wb_valid_data_o, wb_bus_en)
		begin
			
			-- default conditions

			wb_cyc_i <= '0';
			wb_stb_i <= '0';
			busy <= '0';
			state_next <= state_reg;
	


			case state_reg is
				
				-- initial state 	
				when ready =>
				

					wb_cyc_i <= '0';
					wb_stb_i <= '0';
					busy <= '0';

					if(wb_bus_en = '1') then
						state_next <= edge_0;
					else
						state_next <= state_reg;
					end if;
					
				--  raise cyc, stb, we and put in address and data	
				when edge_0 =>

					wb_cyc_i <= '1';
					wb_stb_i <= '1';
					busy <= '1';
					wb_adr_i(7 downto 0) <= wb_valid_address(7 downto 0);
					
					
					if(w_enable = '0') then
						wb_we_i <= '0';					-- set for read
					else
						wb_we_i <= '1';					-- set for write
						wb_dat_i(7 downto 0) <= wb_valid_data(7 downto 0);					
					end if;
				
					state_next <= edge_1;
					
				-- maintain signals for another clock edge
				when edge_1 =>

					wb_cyc_i <= '1';																									-- keep cyc and stb high
					wb_stb_i <= '1';
					busy <= '1';
					wb_adr_i(7 downto 0) <= wb_valid_address(7 downto 0);			-- latch in valid address
					
					
					if(w_enable = '0') then
						wb_we_i <= '0';					-- set for read
						if(wb_ack_o = '1') then
							state_next <= edge_2;
						else
							state_next <= state_reg;
						end if;
						
					else
						wb_we_i <= '1';					-- set for write
						wb_dat_i(7 downto 0) <= wb_valid_data(7 downto 0);				-- latch in valid data		
						-- wait for wish bone ack and then go to ready state
						if(wb_ack_o = '1') then
							state_next <= edge_2;
						else
							state_next <= state_reg;
						end if;
					end if;

				
				-- wait for the wb_ack_o signal to go back to ready and drive everything low
				when edge_2 =>
					
					busy <= '1';

					wb_adr_i(7 downto 0) <= wb_valid_address(7 downto 0);
					-- If write not enabled then read
					if(w_enable = '0') then
						wb_cyc_i <= '1';																							-- keep cyc and stb high
						wb_stb_i <= '1';					
						wb_we_i <= '0';																							-- set for read
						wb_dat_o(7 downto 0) <= wb_valid_data_o(7 downto 0);
						state_next <= ready;																				-- done with cycle so go to ready state
					-- If write enabled then write	
					else
						wb_cyc_i <= '1';																							-- keep cyc and stb high
						wb_stb_i <= '1';		
						wb_we_i <= '1';																							-- set for write
					    wb_dat_i(7 downto 0) <= wb_valid_data(7 downto 0);			-- latch in valid data
						-- wait for wish bone ack and then go to ready state
							state_next <= ready;																			-- go to ready state when ack is high
					end if;
	
			end case;	
			
	end process;

																		
end arch;
